@@ -1,24 +1,31 @@
 /*!
 A high-level, safe, zero-allocation TrueType font parser.
 
+Supports [TrueType](https://docs.microsoft.com/en-us/typography/truetype/),
+[OpenType](https://docs.microsoft.com/en-us/typography/opentype/spec/)
+and [AAT](https://developer.apple.com/fonts/TrueType-Reference-Manual/RM06/Chap6AATIntro.html)
+fonts.
+
 Font parsing starts with a [`Face`].
 
 ## Features
 
-- A high-level API for most common features, hiding all parsing and data resolving logic.
-- A low-level API for for some complex TrueType tables, still hiding raw data layout.
+- A high-level API for most common properties, hiding all parsing and data resolving logic.
+- A low-level, but safe API to access TrueType tables data.
+- Highly configurable. You can disable most of the features, reducing binary size.
+  You can also parse TrueType tables separately, without loading the whole font/face.
 - Zero heap allocations.
 - Zero unsafe.
 - Zero dependencies.
 - `no_std`/WASM compatible.
 - Fast.
-- Stateless. All parsing methods are immutable methods.
+- Stateless. All parsing methods are immutable.
 - Simple and maintainable code (no magic numbers).
 
 ## Safety
 
 - The library must not panic. Any panic considered as a critical bug and should be reported.
-- The library forbids the unsafe code.
+- The library forbids unsafe code.
 - No heap allocations, so crash due to OOM is not possible.
 - All recursive methods have a depth limit.
 - Technically, should use less than 64KiB of stack in worst case scenario.
@@ -656,10 +663,10 @@ impl<'a> Face<'a> {
         let mut s = Stream::new(data);
 
         // Read **font** magic.
-        let magic: Magic = s.read().ok_or(FaceParsingError::UnknownMagic)?;
+        let magic = s.read::<Magic>().ok_or(FaceParsingError::UnknownMagic)?;
         if magic == Magic::FontCollection {
             s.skip::<u32>(); // version
-            let number_of_faces: u32 = s.read().ok_or(FaceParsingError::MalformedFont)?;
+            let number_of_faces = s.read::<u32>().ok_or(FaceParsingError::MalformedFont)?;
             let offsets = s.read_array32::<Offset32>(number_of_faces)
                 .ok_or(FaceParsingError::MalformedFont)?;
 
@@ -672,14 +679,14 @@ impl<'a> Face<'a> {
 
             // Read **face** magic.
             // Each face in a font collection also starts with a magic.
-            let magic: Magic = s.read().ok_or(FaceParsingError::UnknownMagic)?;
+            let magic = s.read::<Magic>().ok_or(FaceParsingError::UnknownMagic)?;
             // And face in a font collection can't be another collection.
             if magic == Magic::FontCollection {
                 return Err(FaceParsingError::UnknownMagic);
             }
         }
 
-        let num_tables: u16 = s.read().ok_or(FaceParsingError::MalformedFont)?;
+        let num_tables = s.read::<u16>().ok_or(FaceParsingError::MalformedFont)?;
         s.advance(6); // searchRange (u16) + entrySelector (u16) + rangeShift (u16)
         let table_records = s.read_array16::<TableRecord>(num_tables)
             .ok_or(FaceParsingError::MalformedFont)?;
@@ -1042,10 +1049,15 @@ impl<'a> Face<'a> {
         }
 
         let mut value = self.tables.hhea.line_gap;
-        if value == 0 {
+        // For line gap, we have to check that ascender or descender are 0, not line gap itself.
+        if self.tables.hhea.ascender == 0 || self.tables.hhea.descender == 0 {
             if let Some(os_2) = self.tables.os2 {
-                value = os_2.typographic_line_gap();
-                value = self.apply_metrics_variation(Tag::from_bytes(b"hlgp"), value);
+                if os_2.typographic_ascender() != 0 || os_2.typographic_descender() != 0 {
+                    value = os_2.typographic_line_gap();
+                    value = self.apply_metrics_variation(Tag::from_bytes(b"hlgp"), value);
+                } else {
+                    value = 0;
+                }
             }
         }
 
